@@ -1,5 +1,9 @@
 stConstruct = function(x, space, time, SpatialObj = NULL, 
 		TimeObj = NULL, crs = CRS(as.character(NA))) {
+	if (is(x, "xts")) {
+		stopifnot(ncol(x) == length(space))
+		return(STFDF(space, index(x), data.frame(x = as.vector(t(x)))))
+	}
 	if (is(x, "matrix"))
 		x = data.frame(x)
 	stopifnot(is(x, "data.frame"))
